/*************************************************************************************
 *  Copyright (C) 2008-2011 by Aleix Pol <aleixpol@kde.org>                          *
 *  Copyright (C) 2008-2011 by Alex Fiestas <alex@eyeos.org>                         *
 *                                                                                   *
 *  This program is free software; you can redistribute it and/or                    *
 *  modify it under the terms of the GNU General Public License                      *
 *  as published by the Free Software Foundation; either version 2                   *
 *  of the License, or (at your option) any later version.                           *
 *                                                                                   *
 *  This program is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
 *  GNU General Public License for more details.                                     *
 *                                                                                   *
 *  You should have received a copy of the GNU General Public License                *
 *  along with this program; if not, write to the Free Software                      *
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA   *
 *************************************************************************************/

#ifndef WHITEWIDGETMANAGER_H
#define WHITEWIDGETMANAGER_H

#include <QObject>
#include <QDesktopWidget>
#include "whitewidget.h"

class QPropertyAnimation;
class WhiteWidgetManager : public QObject
{
    Q_OBJECT
    Q_PROPERTY(qreal opacity READ opacity WRITE setOpacity)
    public:
        WhiteWidgetManager(QObject* parent);
        ~WhiteWidgetManager();

        qreal opacity() const;
        void setOpacity(qreal op);

    public Q_SLOTS:
        void showAll();
        void hideAll();

    private:
        void createWhiteWidgets();
        QList<WhiteWidget*> whitewidgetList;
        QPropertyAnimation* m_timer;
};

#endif //WHITEWIDGETMANAGER
